@@ -1,6 +1,7 @@
 /* pqpath.c - single path into libpq
  *
  * Copyright (C) 2003-2019 Federico Di Gregorio <fog@debian.org>
+ * Copyright (C) 2020-2021 The Psycopg Team
  *
  * This file is part of psycopg.
  *
@@ -46,6 +47,7 @@
 #include "psycopg/libpq_support.h"
 #include "libpq-fe.h"
 
+#include <stdlib.h>
 #ifdef _WIN32
 /* select() */
 #include <winsock2.h>
@@ -53,6 +55,8 @@
 #include "win32_support.h"
 #elif defined(__sun) && defined(__SVR4)
 #include "solaris_support.h"
+#elif defined(_AIX)
+#include "aix_support.h"
 #else
 #include <sys/time.h>
 #endif
@@ -344,14 +348,19 @@ pq_begin_locked(connectionObject *conn, PyThreadState **tstate)
     char buf[256];  /* buf size must be same as bufsize */
     int result;
 
-    Dprintf("pq_begin_locked: pgconn = %p, autocommit = %d, status = %d",
+    Dprintf("pq_begin_locked: pgconn = %p, %d, status = %d",
             conn->pgconn, conn->autocommit, conn->status);
 
-    if (conn->autocommit || conn->status != CONN_STATUS_READY) {
+    if (conn->status != CONN_STATUS_READY) {
         Dprintf("pq_begin_locked: transaction in progress");
         return 0;
     }
 
+    if (conn->autocommit && !conn->entered) {
+        Dprintf("pq_begin_locked: autocommit and no with block");
+        return 0;
+    }
+
     if (conn->isolevel == ISOLATION_LEVEL_DEFAULT
             && conn->readonly == STATE_DEFAULT
             && conn->deferrable == STATE_DEFAULT) {
@@ -390,10 +399,10 @@ pq_commit(connectionObject *conn)
     Py_BEGIN_ALLOW_THREADS;
     pthread_mutex_lock(&conn->lock);
 
-    Dprintf("pq_commit: pgconn = %p, autocommit = %d, status = %d",
-            conn->pgconn, conn->autocommit, conn->status);
+    Dprintf("pq_commit: pgconn = %p, status = %d",
+            conn->pgconn, conn->status);
 
-    if (conn->autocommit || conn->status != CONN_STATUS_BEGIN) {
+    if (conn->status != CONN_STATUS_BEGIN) {
         Dprintf("pq_commit: no transaction to commit");
         retvalue = 0;
     }
@@ -424,10 +433,10 @@ pq_abort_locked(connectionObject *conn, PyThreadState **tstate)
 {
     int retvalue = -1;
 
-    Dprintf("pq_abort_locked: pgconn = %p, autocommit = %d, status = %d",
-            conn->pgconn, conn->autocommit, conn->status);
+    Dprintf("pq_abort_locked: pgconn = %p, status = %d",
+            conn->pgconn, conn->status);
 
-    if (conn->autocommit || conn->status != CONN_STATUS_BEGIN) {
+    if (conn->status != CONN_STATUS_BEGIN) {
         Dprintf("pq_abort_locked: no transaction to abort");
         return 0;
     }
@@ -485,12 +494,12 @@ pq_reset_locked(connectionObject *conn, PyThreadState **tstate)
 {
     int retvalue = -1;
 
-    Dprintf("pq_reset_locked: pgconn = %p, autocommit = %d, status = %d",
-            conn->pgconn, conn->autocommit, conn->status);
+    Dprintf("pq_reset_locked: pgconn = %p, status = %d",
+            conn->pgconn, conn->status);
 
     conn->mark += 1;
 
-    if (!conn->autocommit && conn->status == CONN_STATUS_BEGIN) {
+    if (conn->status == CONN_STATUS_BEGIN) {
         retvalue = pq_execute_command_locked(conn, "ABORT", tstate);
         if (retvalue != 0) return retvalue;
     }
