@@ -3,7 +3,7 @@
 # test_connection.py - unit test for connection attributes
 #
 # Copyright (C) 2008-2019 James Henstridge  <james@jamesh.id.au>
-# Copyright (C) 2020 The Psycopg Team
+# Copyright (C) 2020-2021 The Psycopg Team
 #
 # psycopg2 is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published
@@ -42,9 +42,10 @@ import psycopg2.extras
 from psycopg2 import extensions as ext
 
 from .testutils import (
-    PY2, unittest, skip_if_no_superuser, skip_before_postgres,
+    unittest, skip_if_no_superuser, skip_before_postgres,
     skip_after_postgres, skip_before_libpq, skip_after_libpq,
-    ConnectingTestCase, skip_if_tpc_disabled, skip_if_windows, slow)
+    ConnectingTestCase, skip_if_tpc_disabled, skip_if_windows, slow,
+    skip_if_crdb, crdb_version)
 
 from .testconfig import dbhost, dsn, dbname
 
@@ -74,6 +75,7 @@ class ConnectionTests(ConnectingTestCase):
         conn.close()
         self.assertEqual(curs.closed, True)
 
+    @skip_if_crdb("backend pid")
     @skip_before_postgres(8, 4)
     @skip_if_no_superuser
     @skip_if_windows
@@ -88,6 +90,7 @@ class ConnectionTests(ConnectingTestCase):
         conn.close()
         self.assertEqual(conn.closed, 1)
 
+    @skip_if_crdb("isolation level")
     def test_reset(self):
         conn = self.conn
         # switch session characteristics
@@ -111,6 +114,7 @@ class ConnectionTests(ConnectingTestCase):
         if self.conn.info.server_version >= 90100:
             self.assert_(conn.deferrable is None)
 
+    @skip_if_crdb("notice")
     def test_notices(self):
         conn = self.conn
         cur = conn.cursor()
@@ -120,6 +124,7 @@ class ConnectionTests(ConnectingTestCase):
         self.assertEqual("CREATE TABLE", cur.statusmessage)
         self.assert_(conn.notices)
 
+    @skip_if_crdb("notice")
     def test_notices_consistent_order(self):
         conn = self.conn
         cur = conn.cursor()
@@ -140,6 +145,7 @@ class ConnectionTests(ConnectingTestCase):
         self.assert_('table4' in conn.notices[3])
 
     @slow
+    @skip_if_crdb("notice")
     def test_notices_limited(self):
         conn = self.conn
         cur = conn.cursor()
@@ -154,6 +160,7 @@ class ConnectionTests(ConnectingTestCase):
         self.assert_('table99' in conn.notices[-1], conn.notices[-1])
 
     @slow
+    @skip_if_crdb("notice")
     def test_notices_deque(self):
         conn = self.conn
         self.conn.notices = deque()
@@ -184,6 +191,7 @@ class ConnectionTests(ConnectingTestCase):
         self.assertEqual(len([n for n in conn.notices if 'CREATE TABLE' in n]),
             100)
 
+    @skip_if_crdb("notice")
     def test_notices_noappend(self):
         conn = self.conn
         self.conn.notices = None    # will make an error swallowes ok
@@ -230,13 +238,14 @@ class ConnectionTests(ConnectingTestCase):
         self.assert_(time.time() - t0 < 7,
             "something broken in concurrency")
 
+    @skip_if_crdb("encoding")
     def test_encoding_name(self):
         self.conn.set_client_encoding("EUC_JP")
         # conn.encoding is 'EUCJP' now.
         cur = self.conn.cursor()
         ext.register_type(ext.UNICODE, cur)
         cur.execute("select 'foo'::text;")
-        self.assertEqual(cur.fetchone()[0], u'foo')
+        self.assertEqual(cur.fetchone()[0], 'foo')
 
     def test_connect_nonnormal_envvar(self):
         # We must perform encoding normalization at connection time
@@ -293,7 +302,7 @@ class ConnectionTests(ConnectingTestCase):
         # Stop the committer thread
         stop.append(True)
 
-        self.assert_(not notices, "%d notices raised" % len(notices))
+        self.assert_(not notices, f"{len(notices)} notices raised")
 
     def test_connect_cursor_factory(self):
         conn = self.connect(cursor_factory=psycopg2.extras.DictCursor)
@@ -329,11 +338,12 @@ class ConnectionTests(ConnectingTestCase):
         cur = conn.cursor(cursor_factory=None)
         self.assertEqual(type(cur), psycopg2.extras.DictCursor)
 
+    @skip_if_crdb("connect any db")
     def test_failed_init_status(self):
         class SubConnection(ext.connection):
             def __init__(self, dsn):
                 try:
-                    super(SubConnection, self).__init__(dsn)
+                    super().__init__(dsn)
                 except Exception:
                     pass
 
@@ -373,12 +383,11 @@ class ConnectionTests(ConnectingTestCase):
         dir = tempfile.mkdtemp()
         try:
             with open(os.path.join(dir, "mptest.py"), 'w') as f:
-                f.write("""\
-import time
+                f.write(f"""import time
 import psycopg2
 
 def thread():
-    conn = psycopg2.connect(%(dsn)r)
+    conn = psycopg2.connect({dsn!r})
     curs = conn.cursor()
     for i in range(10):
         curs.execute("select 1")
@@ -386,11 +395,11 @@ def thread():
 
 def process():
     time.sleep(0.2)
-""" % {'dsn': dsn})
+""")
 
             script = ("""\
 import sys
-sys.path.insert(0, %(dir)r)
+sys.path.insert(0, {dir!r})
 import time
 import threading
 import multiprocessing
@@ -401,7 +410,7 @@ t.start()
 time.sleep(0.2)
 multiprocessing.Process(target=mptest.process, name='myprocess').start()
 t.join()
-""" % {'dir': dir})
+""".format(dir=dir))
 
             out = sp.check_output(
                 [sys.executable, '-c', script], stderr=sp.STDOUT)
@@ -454,15 +463,12 @@ class ParseDsnTestCase(ConnectingTestCase):
         self.assertTrue(raised, "ProgrammingError raised due to invalid URI")
 
     def test_unicode_value(self):
-        snowman = u"\u2603"
+        snowman = "\u2603"
         d = ext.parse_dsn('dbname=' + snowman)
-        if PY2:
-            self.assertEqual(d['dbname'], snowman.encode('utf8'))
-        else:
-            self.assertEqual(d['dbname'], snowman)
+        self.assertEqual(d['dbname'], snowman)
 
     def test_unicode_key(self):
-        snowman = u"\u2603"
+        snowman = "\u2603"
         self.assertRaises(psycopg2.ProgrammingError, ext.parse_dsn,
             snowman + '=' + snowman)
 
@@ -563,13 +569,21 @@ class IsolationLevelsTestCase(ConnectingTestCase):
 
         conn = self.connect()
         cur = conn.cursor()
+        if crdb_version(conn) is not None:
+            cur.execute("create table if not exists isolevel (id integer)")
+            cur.execute("truncate isolevel")
+            conn.commit()
+            return
+
         try:
             cur.execute("drop table isolevel;")
         except psycopg2.ProgrammingError:
             conn.rollback()
-        cur.execute("create table isolevel (id integer);")
-        conn.commit()
-        conn.close()
+        try:
+            cur.execute("create table isolevel (id integer);")
+            conn.commit()
+        finally:
+            conn.close()
 
     def test_isolation_level(self):
         conn = self.connect()
@@ -581,6 +595,7 @@ class IsolationLevelsTestCase(ConnectingTestCase):
         conn = self.connect()
         self.assert_(conn.encoding in ext.encodings)
 
+    @skip_if_crdb("isolation level")
     def test_set_isolation_level(self):
         conn = self.connect()
         curs = conn.cursor()
@@ -628,6 +643,7 @@ class IsolationLevelsTestCase(ConnectingTestCase):
         curs.execute('show transaction_isolation;')
         self.assertEqual(curs.fetchone()[0], 'serializable')
 
+    @skip_if_crdb("isolation level")
     def test_set_isolation_level_default(self):
         conn = self.connect()
         curs = conn.cursor()
@@ -702,6 +718,7 @@ class IsolationLevelsTestCase(ConnectingTestCase):
         cur1.execute("select count(*) from isolevel;")
         self.assertEqual(1, cur1.fetchone()[0])
 
+    @skip_if_crdb("isolation level")
     def test_isolation_level_read_committed(self):
         cnn1 = self.connect()
         cnn2 = self.connect()
@@ -728,6 +745,7 @@ class IsolationLevelsTestCase(ConnectingTestCase):
         cur1.execute("select count(*) from isolevel;")
         self.assertEqual(2, cur1.fetchone()[0])
 
+    @skip_if_crdb("isolation level")
     def test_isolation_level_serializable(self):
         cnn1 = self.connect()
         cnn2 = self.connect()
@@ -765,6 +783,7 @@ class IsolationLevelsTestCase(ConnectingTestCase):
         self.assertRaises(psycopg2.InterfaceError,
             cnn.set_isolation_level, 1)
 
+    @skip_if_crdb("isolation level")
     def test_setattr_isolation_level_int(self):
         cur = self.conn.cursor()
         self.conn.isolation_level = ext.ISOLATION_LEVEL_SERIALIZABLE
@@ -813,6 +832,7 @@ class IsolationLevelsTestCase(ConnectingTestCase):
         cur.execute("SHOW default_transaction_isolation;")
         self.assertEqual(cur.fetchone()[0], isol)
 
+    @skip_if_crdb("isolation level")
     def test_setattr_isolation_level_str(self):
         cur = self.conn.cursor()
         self.conn.isolation_level = "serializable"
@@ -909,13 +929,22 @@ class ConnectionTwoPhaseTests(ConnectingTestCase):
     def make_test_table(self):
         cnn = self.connect()
         cur = cnn.cursor()
+        if crdb_version(cnn) is not None:
+            cur.execute("CREATE TABLE IF NOT EXISTS test_tpc (data text)")
+            cur.execute("TRUNCATE test_tpc")
+            cnn.commit()
+            cnn.close()
+            return
+
         try:
             cur.execute("DROP TABLE test_tpc;")
         except psycopg2.ProgrammingError:
             cnn.rollback()
-        cur.execute("CREATE TABLE test_tpc (data text);")
-        cnn.commit()
-        cnn.close()
+        try:
+            cur.execute("CREATE TABLE test_tpc (data text);")
+            cnn.commit()
+        finally:
+            cnn.close()
 
     def count_xacts(self):
         """Return the number of prepared xacts currently in the test db."""
@@ -1197,7 +1226,7 @@ class ConnectionTwoPhaseTests(ConnectingTestCase):
 
     def test_xid_unicode(self):
         cnn = self.connect()
-        x1 = cnn.xid(10, u'uni', u'code')
+        x1 = cnn.xid(10, 'uni', 'code')
         cnn.tpc_begin(x1)
         cnn.tpc_prepare()
         cnn.reset()
@@ -1209,10 +1238,10 @@ class ConnectionTwoPhaseTests(ConnectingTestCase):
     def test_xid_unicode_unparsed(self):
         # We don't expect people shooting snowmen as transaction ids,
         # so if something explodes in an encode error I don't mind.
-        # Let's just check uniconde is accepted as type.
+        # Let's just check unicode is accepted as type.
         cnn = self.connect()
         cnn.set_client_encoding('utf8')
-        cnn.tpc_begin(u"transaction-id")
+        cnn.tpc_begin("transaction-id")
         cnn.tpc_prepare()
         cnn.reset()
 
@@ -1240,6 +1269,7 @@ class ConnectionTwoPhaseTests(ConnectingTestCase):
         self.assertEqual(None, xid.bqual)
 
 
+@skip_if_crdb("isolation level")
 class TransactionControlTests(ConnectingTestCase):
     def test_closed(self):
         self.conn.close()
@@ -1483,6 +1513,7 @@ class TestEncryptPassword(ConnectingTestCase):
                 scope=self.conn, algorithm='md5'),
             'md594839d658c28a357126f105b9cb14cfc')
 
+    @skip_if_crdb("password_encryption")
     @skip_before_libpq(10)
     @skip_before_postgres(10)
     def test_encrypt_server(self):
@@ -1648,7 +1679,7 @@ class AutocommitTests(ConnectingTestCase):
 
 class PasswordLeakTestCase(ConnectingTestCase):
     def setUp(self):
-        super(PasswordLeakTestCase, self).setUp()
+        super().setUp()
         PasswordLeakTestCase.dsn = None
 
     class GrassingConnection(ext.connection):
@@ -1668,6 +1699,7 @@ class PasswordLeakTestCase(ConnectingTestCase):
                 # the password away
                 PasswordLeakTestCase.dsn = self.dsn
 
+    @skip_if_crdb("connect any db")
     def test_leak(self):
         self.assertRaises(psycopg2.DatabaseError,
             self.GrassingConnection, "dbname=nosuch password=whateva")
@@ -1700,8 +1732,7 @@ class SignalTestCase(ConnectingTestCase):
             """)
 
     def _test_bug_551(self, query):
-        script = ("""\
-import os
+        script = f"""import os
 import sys
 import time
 import signal
@@ -1722,7 +1753,7 @@ def killer():
 
 signal.signal(signal.SIGABRT, handle_sigabort)
 
-conn = psycopg2.connect(%(dsn)r)
+conn = psycopg2.connect({dsn!r})
 
 cur = conn.cursor()
 
@@ -1733,8 +1764,8 @@ t.daemon = True
 t.start()
 
 while True:
-    cur.execute(%(query)r, ("Hello, world!",))
-""" % {'dsn': dsn, 'query': query})
+    cur.execute({query!r}, ("Hello, world!",))
+"""
 
         proc = sp.Popen([sys.executable, '-c', script],
             stdout=sp.PIPE, stderr=sp.PIPE)
@@ -1853,6 +1884,7 @@ class TestConnectionInfo(ConnectingTestCase):
         self.assert_(self.conn.info.socket >= 0)
         self.assert_(self.bconn.info.socket < 0)
 
+    @skip_if_crdb("backend pid")
     def test_backend_pid(self):
         cur = self.conn.cursor()
         try:
@@ -1896,11 +1928,16 @@ class TestConnectionInfo(ConnectingTestCase):
                 self.assertIsInstance(self.conn.info.ssl_attribute(attrib), str)
         else:
             for attrib in attribs:
+                # Behaviour changed in PostgreSQL 15
+                if attrib == "library":
+                    continue
                 self.assertIsNone(self.conn.info.ssl_attribute(attrib))
 
         self.assertIsNone(self.conn.info.ssl_attribute('wat'))
 
         for attrib in attribs:
+            if attrib == "library":
+                continue
             self.assertIsNone(self.bconn.info.ssl_attribute(attrib))
 
 
